/*
 * Projet LO43
 * Julien Miltat -- Julien Rosener -- Thomas Petazzoni
 * Semestre automne 2002-2003
 */

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import java.util.*;

public class EnvironnementPanel extends JPanel implements ActionListener
{
  private int sizex, sizey;
  BufferedImage offscreen;
  Graphics goffscreen;

  Color[] initcolors = {
    Color.green,
    Color.red,
    Color.cyan,
    Color.orange
  };

  Hashtable colorHashtable;
  Random  r;
  javax.swing.Timer t;
  Simulation s;
  Interface iface;

  EnvironnementPanel(int x, int y, int nbfourm, int foodmax, int foodmin, 
		     int foodtotal, int obsttotal, double evapphero, 
		     double zerophero, Interface iface0)
  {
    super();

    // Initialisation des champs
    sizex      = x;
    sizey      = y;
    iface      = iface0;
    setSize(x*2+2, y*2+2);
    setVisible(true);

    offscreen      = new BufferedImage(getWidth(), getHeight(), 
				       BufferedImage.TYPE_INT_RGB);
    colorHashtable = new Hashtable();
    r              = new Random();

    if(offscreen == null)
      System.out.println("OUUUPPPPS : " + getWidth() + "x" + getHeight());

    // Preparation de l'image
    goffscreen = offscreen.getGraphics();
    goffscreen.setColor(Color.black);
    goffscreen.fillRect(1, 1, getWidth()-2, getHeight()-2);
    goffscreen.setColor(Color.white);
    goffscreen.drawRect(0, 0, getWidth()-1, getHeight()-1);	

    // Creation de l'object Simulation
    s = new Simulation(x, y, foodmax, foodmin, foodtotal, obsttotal, 
		       evapphero, zerophero, this);

    // Creation des fourmilieres initiales
    for (int i = 0; i < nbfourm; i++)
      addFourmiliere();

    // Lancement de la simulation
    t = new javax.swing.Timer(100, this);
    t.start();
  }

  public void stop()
  {
    simuStop();
    s.destroy();
  }
    
  public void simuStop()
  {
    t.stop();
  }
    
  public void simuStart()
  {
    t.start();
  }

  private int convertCoord(int i)
  {
    return i*2+1;
  }

  public void update(Graphics g)
  {
    paint(g);
  }

  public void paint(Graphics g) 
  {
    super.paint(g);
    g.drawImage(offscreen, 0, 0, this);
  }

  public void actionPerformed(ActionEvent e) {
    String txt = "";

    s.simu();

    for (int i = 0; i < s.getNbFourmiliere(); i++)
      {
	txt += 
	  "Fourmiliere " + i + " :\n" + 
	    "   Food : " + s.getFourmiliereFood(i) + "\n" +
	    "   NbFourmis : " + s.getFourmiliereNbrFourmis(i) + "\n\n";
      }

    iface.changeText(txt);
	
    repaint();
  }

  public void addFourmiliere()
  {
    int idx = s.addFourmiliere();

    /* Si on a une couleur predefinie, on l'utilise */
    if(idx < 4)
      {
	colorHashtable.put(new Integer(idx), initcolors[idx]);
      }
    /* Sinon il faut generer une couleur aleatoirement */
    else
      {
	int notgood = 1;
	int red = 0;
	int green = 0;
	int blue = 0;

	/* On essaie de trouver une couleur qui nous convient */
	while (notgood != 0)
	  {
	    notgood = 0;

	    /* Generation aleatoire de la couleur */
	    red   = r.nextInt(256);
	    green = r.nextInt(256);
	    blue  = r.nextInt(256);
			
	    /* Est-ce qu'on est trop pret du blanc ? */
	    if((Math.pow(red-255,2) + 
		Math.pow(green-255,2) + 
		Math.pow(blue-255,2)) 
	       < 500)
	      notgood = 1;

	    /* Est-ce qu'on est trop pret du noir ? */
	    else if((Math.pow(red-0,2) + 
		     Math.pow(green-0,2) + 
		     Math.pow(blue-0,2)) 
		    < 500)
	      notgood = 1;
			
	    /* Est-ce qu'on est trop pret de la couleur
	       d'une fourmiliere deja existante ? */
	    else
	      {
		Enumeration enum = colorHashtable.keys();
		while(enum.hasMoreElements())
		  {
		    Color c = (Color) colorHashtable.get(enum.nextElement());
		    if((Math.pow(red-c.getRed(), 2) +
			Math.pow(green-c.getGreen(), 2) + 
			Math.pow(blue-c.getBlue(), 2)) < 300)
		      notgood = 1;
		  }
	      }
			
	  }
	colorHashtable.put(new Integer(idx), new Color(red, green, blue));
      }
  }
    
  public void paintFourmi(int x, int y, int nf)
  {
    goffscreen.setColor((Color) colorHashtable.get(new Integer(nf)));
    goffscreen.fillRect(convertCoord(x), convertCoord(y), 2, 2);
  }
    
  public void clearFourmi(int x, int y, int nf)
  {
    goffscreen.setColor(Color.black);
    goffscreen.fillRect(convertCoord(x), convertCoord(y), 2, 2);
  }
    
  public void paintObst(int x, int y)
  {
    goffscreen.setColor(Color.blue);
    goffscreen.fillRect(convertCoord(x), convertCoord(y), 2, 2);
  }

  public void paintFood(int x, int y)
  {
    goffscreen.setColor(Color.yellow);
    goffscreen.fillRect(convertCoord(x), convertCoord(y), 2, 2);
  }

  public void clearFood(int x, int y)
  {
    goffscreen.setColor(Color.black);
    goffscreen.fillRect(convertCoord(x), convertCoord(y), 2, 2);
  }
    
  public void paintPhero(int x, int y, int nf)
  {
    goffscreen.setColor((Color) colorHashtable.get(new Integer(nf)));
    goffscreen.fillRect(convertCoord(x), convertCoord(y), 1, 1);
  }
  
  public void clearPhero(int x, int y, int nf)
  {
    goffscreen.setColor(Color.black);
    goffscreen.fillRect(convertCoord(x), convertCoord(y), 1, 1);
  }
  
  public void paintFourmiliere(int x, int y, int nf)
  {
    goffscreen.setColor((Color) colorHashtable.get(new Integer(nf)));
    goffscreen.fillRect(convertCoord(x), convertCoord(y), 4, 4);
  }

  public void clearFourmiliere(int x, int y, int nf)
  {
    goffscreen.setColor(Color.black);
    goffscreen.fillRect(convertCoord(x), convertCoord(y), 4, 4);
  }
}
