/*
 * Projet LO43
 * Julien Miltat -- Julien Rosener -- Thomas Petazzoni
 * Semestre automne 2002-2003
 */

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <iostream.h>

#include "environnement.h"
#include "fourmi.h"
#include "fourmiliere.h"

int dirX[8]   = {  0, -1, -1, -1,  0,  1,  1,  1 };
int dirY[8]   = { -1, -1,  0,  1,  1,  1,  0, -1 };
int dirInv[8] = {  4,  5,  6,  7,  0,  1,  2,  3 };

environnement::environnement (int szx, int szy,int foodmax, 
			      int foodmin, int foodtotal, int obsttotal,
			      float evapphero0, float zerophero0, 
			      mygraphics & g0) : g(g0)
{
  srand48 (time(NULL)) ;
  
  sizex            = szx;
  sizey            = szy;
  evapphero        = evapphero0;
  zerophero        = zerophero0;
  fourmiliereident = 0;
  nbfourmiliere    = 0;


  /* Allocation de la grille : checker le retour de malloc ? */
  grille = (int **) malloc(sizex * sizeof(int*));
  if(grille == NULL)
    {
      fprintf(stderr, "ERREUR ALLOCATION\n");
      exit(1);
    }
  
  for (int i = 0; i < sizex; i++)
    {
      grille[i] = (int *) malloc(sizey * sizeof(int));
      if(grille[i] == NULL)
	{
	  fprintf(stderr, "ERREUR ALLOCATION\n");
	  exit(1);
	}
    }
  

  /* Initialisation de la grille */
  for (int i = 0 ; i < sizex ; i++)
    for (int j = 0 ; j < sizey ; j++) 
      grille[i][j] = 0 ;	  

  /* Generation des obstacles */
  for (int i = 0 ; i < (obsttotal / 10) ; i++) 
    {
      int x = (int) (drand48()*(sizex-5)+5) ;
      int y = (int) (drand48()*(sizey-5)+5) ;

      for (int j = 0 ; j < 20 ; j++) 
	{
	  int x0 = (int) (x + drand48() * 6 - 3);
	  int y0 = (int) (y + drand48() * 6 - 3);
	  if(x0 < sizex-1 && y0 < sizey-1 && x0 > 0 && y0 > 0)
	    {
	      showObst(x0, y0);
	      grille[x0][y0] = -1;
	    }
	}
    }

  /* Generation de la nourriture */
  for (int i = 0 ; i < foodtotal ; i++) 
    {
      int x = (int) (drand48() * sizex);
      int y = (int) (drand48() * sizey);
      grille[x][y] += (int) (drand48()*(foodmax-foodmin)+foodmin);
      showFood(x, y);
    }

  /* Aucune fourmiliere pour le moment */
  liste_fourmilieres = NULL;
}

environnement::~environnement()
{
  for (int i = 0; i < nbfourmiliere; i++)
    delete(liste_fourmilieres[i]);
  
  free(liste_fourmilieres);

  for (int i = 0; i < sizex; i++)
    free(grille[i]);

  free(grille);
}


int environnement::addFourmiliere()
{
  fourmiliere **newlist;
  int hasstuffnear = 1;
  int x0, y0;

  /* Recherche d'une position pour la fourmiliere */
  while(hasstuffnear != 0)
    {
      hasstuffnear = 0;
      
      x0 = (int) ((drand48() * sizex));
      y0 = (int) ((drand48() * sizey));

      if(x0-4 < 0 || y0-4 < 0 || x0+4 > sizex || y0+4 > sizey)
	hasstuffnear = 1;

      /* Detection de la proximite d'un osbtacle */
      for (int obsx = -2; obsx < 2 && !hasstuffnear; obsx++)
	for (int obsy = -2; obsy < 2 && !hasstuffnear; obsy++)
	  if((x0+obsx) > 0 && (y0+obsy) > 0 && 
	     (x0+obsx) < sizex && (y0+obsy) < sizey && 
	     grille[x0+obsx][y0+obsy] == -1)
	    hasstuffnear = 1;
      
      /* Detection de la proximite d'une fourmiliere */
      for (int obsx = -(sizex/10); obsx < (sizex/10) && !hasstuffnear; obsx++)
	for (int obsy = -(sizey/10); obsy < (sizey/10) && !hasstuffnear; obsy++)
	  if((x0+obsx) > 0 && (y0+obsy) > 0 && 
	     (x0+obsx) < sizex && (y0+obsy) < sizey && 
	     grille[x0+obsx][y0+obsy] == -2)
	    hasstuffnear = 1;
    }

  /* Marquage de la zone de la fourmiliere */
  for (int xi = 0; xi < 2; xi++)
    for (int yi = 0; yi < 2; yi++)
      grille[x0+xi][y0+yi] = -2;

  nbfourmiliere++;
  
  /* Allocation du nouveau tableau */
  newlist = (fourmiliere **) malloc(nbfourmiliere * sizeof(fourmiliere*));
  if(newlist == NULL)
    {
      fprintf(stderr, "ERREUR ALLOCATION\n");
      exit(1);
    }

  
  for (int i = 0; i < (nbfourmiliere-1); i++)
    newlist[i] = liste_fourmilieres[i];

  /* Insertion de la nouvelle fourmiliere */
  newlist[nbfourmiliere-1] = new fourmiliere(x0, y0, fourmiliereident++,
					     sizex, sizey,
					     evapphero, zerophero,
					     this);

  free(liste_fourmilieres);
  liste_fourmilieres = newlist;

  showFourmiliere(x0, y0, fourmiliereident-1);

  return fourmiliereident-1;
}

int environnement::delFourmiliere(int i)
{
  return 0;
}

/** Indique si la direction indiquee a partir du point courant est
    possible (pas d'obstacle) */
int environnement::isDirPossible (int x, int y, int d)
{
  if ((d>=0) && (d<8)) 
    {
      x += dirX[d] ;
      y += dirY[d] ;
      if ((x >= 0) && (x < sizex) && (y >= 0) && (y < sizey)) 
	{
	  if (grille[x][y] >= 0) 
	    return 1;
	}
    }

  return 0;
}

/** Lance la simulation de l'ensemble des fourmilieres */
void environnement::simu () 
{  
  for (int i = 0; i < nbfourmiliere; i++)
    {
      liste_fourmilieres[i]->simu();
    }
}

/** Retourne dans quelle direction on peut aller pour trouver de la
    nourriture */
int environnement::isFoodNear (int x0, int y0) 
{
  int food = 0, dir = -1;
  
  for (int d=0;d<8;d++) 
    {
      int x = x0 + dirX[d];
      int y = y0 + dirY[d];
      
      if ((x>=0) && (y>=0) && (x < sizex) && (y < sizey)) 
	{
	  if (grille[x][y] > food) 
	    {
	      food = grille[x][y];
	      dir = d;
	    }
	}
    }
  
  return dir ;
}

/** Retourne un peu de food, situee a la case (x0, y0) dans la limite
    de fmax food */
int environnement::getFood (int x0, int y0, int fmax) 
{
    int food;
  
  /* Y-a-t'il a manger ? */
  if (grille[x0][y0]>0) 
    {
      /* Y'a-t-il suffisamment a manger, si oui on prend fmax de food */
      if(grille[x0][y0] - fmax >= 0)
	{
	  grille[x0][y0] -= fmax;
	  food = fmax;
	}
      /* Il n'y a pas sufisamment a manger, on prend ce qui reste */
      else
	{
	  food = grille[x0][y0];
	  grille[x0][y0] = 0;
	}

      return food;
    }

  return 0 ;
}

/** Ajoute de la nourriture a l'endroit specifie */
void environnement::addFood (int x0, int y0, int f) 
{
  if (grille[x0][y0]>=0) 
    grille[x0][y0] += f;
}


void environnement::repaintCase(int x, int y)
{
    if(grille[x][y] == -1)
	{
	    g.paintObst(x, y);
	}
    else if(grille[x][y] > 0)
	{
	    g.paintFood(x, y);
	}
    else if(grille[x][y] == -2)
	{
	    for (int i = 0; i < nbfourmiliere; i++)
		{
		    if((liste_fourmilieres[i]->getX() == x &&
			liste_fourmilieres[i]->getY() == y) ||
		       (liste_fourmilieres[i]->getX()+1 == x &&
			liste_fourmilieres[i]->getY() == y) ||
		       (liste_fourmilieres[i]->getX() == x &&
			liste_fourmilieres[i]->getY()+1 == y) ||
		       (liste_fourmilieres[i]->getX()+1 == x &&
			liste_fourmilieres[i]->getY()+1 == y))
			{
			    g.paintFourmiliere(liste_fourmilieres[i]->getX(), 
					       liste_fourmilieres[i]->getY(), 
					       liste_fourmilieres[i]->getFourmiliereId());
			}
		}
	}
    
    /* Reaffichage des pheromones */
    for (int i = 0; i < nbfourmiliere; i++)
	{
	    if(liste_fourmilieres[i]->hasPhero(x, y))
		showPhero(x, y, liste_fourmilieres[i]->getFourmiliereId());
	}    
}

void environnement::showFourmi(int x, int y, int nf) 
{
  g.paintFourmi(x, y, nf);
}

void environnement::unshowFourmi(int x, int y, int nf)
{
  g.clearFourmi(x, y, nf);

  if(grille[x][y] == -1)
    {
      cerr << "Une fourmi a marche sur un obstacle (" << x 
	   << "," << y << ")" << endl;
      exit(1);
    }
  
  repaintCase(x, y);
}

void environnement::showFood(int x, int y)
{
    g.paintFood(x, y);
}

void environnement::unshowFood(int x, int y)
{
    g.clearFood(x, y);
    repaintCase(x, y);
}

void environnement::showObst(int x, int y)
{
    g.paintObst(x, y);
}

void environnement::showPhero(int x, int y, int nf)
{
  g.paintPhero(x, y, nf);
}

void environnement::unshowPhero(int x, int y, int nf)
{
    g.clearPhero(x, y, nf);
    repaintCase(x, y);
}

void environnement::showFourmiliere(int x, int y, int nf)
{
    g.paintFourmiliere(x, y, nf);
}

void environnement::unshowFourmiliere(int x, int y, int nf)
{
    g.clearFourmiliere(x, y, nf);
    repaintCase(x, y);
}








