/*
 * Projet LO43
 * Julien Miltat -- Julien Rosener -- Thomas Petazzoni
 * Semestre automne 2002-2003
 */

#ifndef _ENVIRONNEMENT_H_
#define _ENVIRONNEMENT_H_

#include "graphics.h"

class fourmiliere;

extern int dirX[8] ;
extern int dirY[8] ;
extern int dirInv[8] ;

class environnement {
private:
  /** Taille du monde (abcisse) */
  int sizex;

  /** Taille du monde (ordonnee) */
  int sizey;

  /** Nombre de fourmilieres */
  int nbfourmiliere;

  /** Numero de la derniere fourmiliere cree (pour les identifier de
      maniere unique) */
  int fourmiliereident;
  
  /** Tableau bidimensionnel representant le monde. Les
      cases avec -2 sont occupees par une fourmiliere, les cases
      avec -1 sont des obstacles, les cases avec 0 sont libres, et
      les valeurs positives indiquent une presence de nourriture. */
  int **grille;
    
  /** Tableau de pointeurs vers les differentes fourmilieres */
  fourmiliere **liste_fourmilieres;

  /** Parametres concernant les pheromones, passees a chaque
      fourmiliere lors de leur construction */
  float evapphero, zerophero;

  /** Environnement d'affichage */
  mygraphics & g;

  /* Fonction pour repeindre une case specifiquement */
  void repaintCase(int x, int y);

public:
  environnement (int sizex, int sizey, int foodmax, 
		 int foodmin, int foodtotal, int obsttotal,
		 float evapphero, float zerophero, 
		 mygraphics & g0);
  ~environnement();
  
  inline fourmiliere * getFourmiliere(int i) { return liste_fourmilieres[i]; }
  inline int getGrille(int x,int y) { return grille[x][y]; }
  inline int nbFourmiliere() { return nbfourmiliere; }
  
  int  addFourmiliere();
  int  delFourmiliere(int i);
  int  isDirPossible (int x, int y, int d) ;
  void simu ();
  void makeFourmiliere(int x0, int y0) ;
  int  isFoodNear (int x0, int y0) ;
  int  getFood (int x0, int y0, int fmax) ;
  void addFood (int x0, int y0, int f) ;

  /* Fonction d'affichage "intelligentes" */
  void showFourmi(int x, int y, int nf);
  void unshowFourmi(int x, int y, int nf);
  void showFood(int x, int y);
  void unshowFood(int x, int y);
  void showObst(int x, int y);
  void showPhero(int x, int y, int nf);
  void unshowPhero(int x, int y, int nf);
  void showFourmiliere(int x, int y, int nf);
  void unshowFourmiliere(int x, int y, int nf);
};

#endif

