/*
 * Projet LO43
 * Julien Miltat -- Julien Rosener -- Thomas Petazzoni
 * Semestre automne 2002-2003
 */

#include <stdlib.h>
#include <stdio.h>
#include <iostream.h>
#include "fourmi.h"
#include "fourmiliere.h"
#include "environnement.h"
#include "common.h"

int numero_global = 0;

/*
 * Methodes de la classe fourmi
 */
fourmi::fourmi(fourmiliere *f, int x0, int y0)
{
  age        = 10;
  x          = x0;
  y          = y0;
  oldx       = x0;
  oldy       = y0;
  numero     = numero_global++;
  home       = f;
  dirprec    = 0;
  dirprec2   = 0;
}

fourmi::fourmi(fourmi &f)
{
  age        = f.age;
  x          = f.x;
  y          = f.y;
  oldx       = f.oldx;
  oldy       = f.oldy;
  numero     = numero_global++;
  home       = f.home;
}

int fourmi::numeroFourmi()
{
  return numero;
}

ostream& operator<< (ostream& o, fourmi& f)
{
  f.affiche();
  return o;
}

/*
 * Reine
 */
Reine::Reine(fourmiliere *h, int x, int y) : fourmi(h,x,y)
{
}

Reine::Reine(Reine &r):fourmi(r)
{
}

ostream& operator<< (ostream& o, Reine& f)
{
  f.affiche();
  return o;
}

void Reine::affiche()
{
  DEBUG_MSG("Reine numero " << numero 
	    << " en position (" << x 
	    << "," << y 
	    << "), (" << oldx
	    << "," << oldy
	    << ") age=" << age);
  return;
}

void Reine::simu()
{

}


/*
 * Ouvriere
 */
Ouvriere::Ouvriere(fourmiliere *h, int x, int y) : fourmi(h,x,y)
{
  nourriture = 0;
  nbrandom   = 0;
  nbobstacle = 0;
  gohome     = 0;
}

Ouvriere::Ouvriere(Ouvriere &o):fourmi(o)
{
}

ostream& operator<< (ostream& o, Ouvriere& f)
{
  f.affiche();
  return o;
}

void Ouvriere::affiche()
{
  DEBUG_MSG("Ouvriere numero " << numero 
	    << " en position (" << x 
	    << "," << y 
	    << "), (" << oldx
	    << "," << oldy
	    << "), age=" << age 
	    << " nourriture=" << nourriture);  
  return;
}

void Ouvriere::simu()
{
  environnement *env = home->getEnv();
  int homex = home->getX();
  int homey = home->getY();
  int direction = -1;

  if(env->getGrille(x, y) > 0)
    {
      nourriture = env->getFood(x, y, 50);
    }  

  if(x == homex && y == homey)
    {
      home->addFood(nourriture);
      nourriture = 0;
      nbrandom   = 0;
      nbobstacle = 0;
      gohome     = 0;
    }

  if(nourriture > 0 && nbrandom == 0 || gohome != 0)
    {
	/* Retourner au nid */
	
	/* On calcule la direction la plus directe pour rentrer au nid */
	if(x < homex)
	  {
	  if(y < homey)
	    direction = 5;
	  else if(y > homey)
	    direction = 7;
	  else
	    direction = 6;
	}
      else if(x > homex)
	{
	  if(y < homey)
	    direction = 3;
	  else if(y > homey)
	    direction = 1;
	  else
	    direction = 2;
	}
      else
	{
	  if(y < homey)
	    direction = 4;
	  else if (y > homey)
	    direction = 0;
	}

      int x1 = x + dirX[direction];
      int y1 = y + dirY[direction];

      /* On regarde si on va dans un obstacle */
      if(env->getGrille(x1, y1) == -1)
	{
	  int encore = 1, d = -1;
	  nbobstacle++;
	  
	  /* On regarde dans les huits directions si on peut eviter */
	  for (d = 0; (d < 8) && (encore == 1); d++)
	    {
	      if(d != dirprec && d != direction)
		{
		  x1 = x + dirX[d];
		  y1 = y + dirY[d];
		  
		  if(env->getGrille(x1, y1) != -1)
		    {
		      encore    = 0;
		      direction = d;
		    }
		}
	    }

	  /* Si on a tape plus de 3 fois dans un obstacle, on se donne
	     10 coups aleatoires pour se decaler avant de repartir
	     direct vers la fourmiliere */
	  if(nbobstacle > 3)
	    {
	      nbobstacle =  0;
	      nbrandom   = 10;
	    }
	}

      /* Laisser de la pheromone */
      if(nourriture > 0)
	home->addPhero(x, y, 1);
    }
  else
    {
      /* Chercher de la food */
      
      /* Y'a-t-il a manger juste autour de moi ? */
      direction = env->isFoodNear(x, y);
      
      /* Il n'y a pas a manger -> on essaie de suivre des phero */
      if(direction == -1)
	{
	  direction = home->isPheroNear(x, y);
	  if(direction != -1)
	    {
	      /* Il y a des pheronomes, mais faut etre sur de pas
		 faire marche arriere, parce que sinon ca veut dire
		 qu'on est au bout de la ligne de phero, et que y'a
		 plus rien a manger ! */
	      if (direction == dirInv[dirprec] && 
		  (((home->getX()-x)*(home->getX()-x))+
		   ((home->getY()-y)*(home->getY()-y))) > 3)
		gohome = 1;
	    }
	}

      /* Il n'y a ni phero, ni bouffe dans les parages -> random */
      if(direction == -1)
	{
	  do
	    {
	      direction = (int) (drand48()*8) ;
	    } while(!env->isDirPossible(x, y, direction));
	}

      /* Si on est ici, c'est parce qu'on etait bloque par un
         obstacble au retour de la bouffe, il faut donc deposer des
         pheromones */
      if(nbrandom > 0)
	{
	  nbrandom--;
	  home->addPhero(x,y,1);
	}
    }
  
  oldx = x;
  oldy = y;
  dirprec = direction;
  
  /* En fonction de la direction, calcul du deplacement */
  /*
  switch(direction)
    {
    case 0:
      y--;
      break;
    case 1: 
      x--;
      y--;
      break;   
    case 2:
      x--;
      break;
    case 3:
      x--;
      y++;
      break;
    case 4:
      y++;
      break;
    case 5:
      x++;
      y++;
      break;
    case 6:
      x++;
      break;
    case 7:
      x++;
      y--;
      break;
    default:
      fprintf(stderr, "Probleme de direction\n");
      exit(1);
    }
  */
  x += dirX[direction];
  y += dirY[direction];

  env->unshowFourmi(oldx, oldy, home->getFourmiliereId());
  env->showFourmi(x, y, home->getFourmiliereId());
}


/*
 * Guerriere
 */
Guerriere::Guerriere(fourmiliere *h, int x, int y) : fourmi(h,x,y)
{
  sens = AVANT;
}

Guerriere::Guerriere(Guerriere &g):fourmi(g)
{
}

ostream& operator<< (ostream& o, Guerriere& f)
{
  f.affiche();
  return o;
}

void Guerriere::affiche()
{
  DEBUG_MSG("Guerriere numero " << numero 
	    << " en position (" << x 
	    << "," << y 
	    << "), (" << oldx
	    << "," << oldy
	    << ") age=" << age);
  return;
}

void Guerriere::simu()
{
  if(oldx == 10)
    sens = ARRIERE;
  else if(oldx == 0)
    sens = AVANT;

  oldx = x;
  oldy = y;

  if(sens == AVANT)
    {
      x++;
      y++;
    }
  else
    {
      x--;
      y--;
    }
}
