/*
 * Projet LO43
 * Julien Miltat -- Julien Rosener -- Thomas Petazzoni
 * Semestre automne 2002-2003
 */
#ifndef __FOURMI_H__
#define __FOURMI_H__

#include <iostream.h>

class fourmiliere;
class environnement;

/*
 * Classe abstraite de base reprsentant une fourmi. Ne peut etre
 * instantiee. Cette classe est heritee au niveau de Reine, Guerriere
 * et 0uvriere, afin de crer des classes pleinement fonctionnelles.
 */
class fourmi 
{
 protected:
  int age;
  int x, y;
  int oldx, oldy;
  int numero;
  int dirprec, dirprec2;
  fourmiliere *home;
public:
  fourmi(fourmiliere *, int x, int y);
  fourmi(fourmi &f);
  friend ostream& operator<< (ostream&, fourmi&);
  virtual void affiche() = 0;
  virtual void simu() = 0;
  int numeroFourmi();
  inline int getOldX() { return oldx;}
  inline int getOldY() {return oldy;}
  inline int getX() {return x;}
  inline int getY() {return y;}
};

/*
 * Guerriere
 */
class Guerriere: public fourmi
{
 private:
  /* Sens de dplacement, utilis au niveau de la simulation */
  enum { AVANT, ARRIERE } sens;
 public:
  Guerriere(fourmiliere *, int x, int y);
  Guerriere(Guerriere &);
  friend ostream& operator<< (ostream&, Guerriere&);
  void affiche();
  void simu();
};

/*
 * Reine
 */
class Reine: public fourmi
{
 public:
  Reine(fourmiliere *, int x, int y);
  Reine(Reine &);
  friend ostream& operator<< (ostream&, Reine&);
  void affiche();
  void simu();
};

/*
 * Ouvriere
 */
class Ouvriere: public fourmi
{
 private:
  /* Quantite de nourriture transportee par l'ouvriere */
  int nourriture;

  /* Donne le nombre de coups aleatoires a faire pour pouvoir se
     sortir d'une situation bloquee */
  int nbrandom;

  /* Nombre de fois que l'on a bute sur un obstacle */
  int nbobstacle;

  /* Booleen indiquant si l'on doit rentrer de force a la
     fourmiliere */
  int gohome;
 public:
  Ouvriere(fourmiliere *, int x, int y);
  Ouvriere(Ouvriere &);
  friend ostream& operator<< (ostream&, Ouvriere&);
  void affiche();
  void simu();
};

/*
 * FourmiElt encapsule un pointeur vers une fourmi et des pointeurs
 * suivant/precendent, afin de pouvoir crer des listes chainees de
 * fourmi
 */
class FourmiElt
{
 public:
  fourmi* f;
  FourmiElt(fourmi &fourmi);

  FourmiElt *suivant;
  FourmiElt *precedent;
};

/*
 * Liste chainee de fourmis
 */
class ListeFourmis
{
 private:
  /* Tete de la liste */
  FourmiElt *tete;

  /* Nombre de fourmis dans la liste */
  int nb_fourmis;

  /* Membre utilise pour le parcours de la liste des fourmis en
     utilisant les methodes getFirst() et getNext() */
  FourmiElt *current;
 public:
  ListeFourmis();
  void ajouter(fourmi& f);
  void supprimer(fourmi& f);
  void affiche();
  int existe(fourmi &f);
  inline int getnbfourmis() { return nb_fourmis; }
  fourmi *get(int i);
  fourmi *getFirst();
  fourmi *getNext();
  FourmiElt* cherche(fourmi &f);
  ListeFourmis& operator+=(fourmi& f);
  ListeFourmis& operator-=(fourmi& f);
  friend ostream& operator<< (ostream&, ListeFourmis&);
};


#endif
