/*
 * Projet LO43
 * Julien Miltat -- Julien Rosener -- Thomas Petazzoni
 * Semestre automne 2002-2003
 */

#include <ctype.h>
#include <stdlib.h>  /* atoi */
#include <unistd.h>
#include "fourmi.h"
#include "environnement.h"
#include "common.h"
#include "fourmiliere.h"

fourmiliere::fourmiliere(int x0, int y0, int nf0, 
			 int sizex0, int sizey0,
			 float tauxevapphero0, float zerophero0,
			 environnement *env0)
{
  /* Creation de la liste chainee de fourmis */
  liste = new ListeFourmis();
  
  /* Initialisation des champs */
  reserveNourriture = 0;
  x                 = x0;
  y                 = y0;
  nf                = nf0;
  nbf               = 0;
  tauxevapphero     = tauxevapphero0;
  zerophero         = zerophero0;
  sizex             = sizex0;
  sizey             = sizey0;
  env               = env0;
  
  /* Allocation du tableau bi-dimensionnel pour la quantite de
     pheromone */
  phero = (float **) malloc(sizex * sizeof(float*));
  for (int i = 0; i < sizex; i++)
    phero[i] = (float *) malloc(sizey * sizeof(float));
  
  /* Initialisation du tableau */
  for (int i = 0; i < sizex; i++)
    for (int j = 0; j < sizey; j++)
      phero[i][j] = 0;

  /* Ouvrieres */
  for (int i = 0; i < 5; i++)
    {
      Ouvriere *o = new Ouvriere(this, x, y);
      ajouter(*o);
    }

  /* Guerrieres */
  for (int i = 0; i < 0; i++)
    {
      Guerriere *o = new Guerriere(this, x, y);
      ajouter(*o);
    }
}

fourmiliere::~fourmiliere()
{
  /* Suppresion de la liste */
  delete(liste);

  /* Liberation du tableau bi-dimensionnel utilise pour representer
     les pheromones */
  for (int i = 0; i < sizex; i++)
    free(phero[i]);
  free(phero);
}

/* Ajout d'une fourmi */
void fourmiliere::ajouter(fourmi &f)
{
  nbf++;
  liste->ajouter(f);
}

/* Suppression d'une fourmi */
void fourmiliere::supprimer(fourmi &f)
{ 
  nbf--;
  liste->supprimer(f);
}

void fourmiliere::affiche()
{ 
  /* On affiche la liste des fourmis */
  if(liste != NULL)
    cout << *liste;
}

ostream& operator<< (ostream& o, fourmiliere& f)
{
  f.affiche();
  return o;
}

#define isnum(c) (('0' <= c) && (c <= '9'))

istream& operator>> (istream& i, fourmiliere& f)
{
  char buffer[256];

  i.get(buffer, 256);
  
  for (int k = 0; buffer[k] != 0 && k < 256; k++)
    if(!isnum(buffer[k]))
      return i;

  int nourriture = atoi(buffer);

  f.reserveNourriture = nourriture;

  return i;
}

void fourmiliere::simu()
{  
  int nb_fourmis = liste->getnbfourmis();

  for (int i = 0; i < nb_fourmis; i++)
    {
      fourmi *f = get(i);
      f->simu();
    }

  for (int x = 0; x < sizex; x++)
    for (int y = 0; y < sizey; y++)
      evapPhero(x, y);  
}

int fourmiliere::isDansNid (int x0, int y0)
{
  int nb_fourmis = liste->getnbfourmis();
  
  for (int i = 0; i < nb_fourmis; i++)
    {
      fourmi *f = get(i);
      if(f->getX() == x0 && f->getY() == y0)
	return 1;
    }
  
  return 0;
}

fourmi *fourmiliere::get(int i)
{
  return liste->get(i);
}

fourmi *fourmiliere::getFirst()
{
  return liste->getFirst();
}

fourmi *fourmiliere::getNext()
{
  return liste->getNext();
}

/*
 * On essaie d'aller vers la case ou il y a le moins de pheromones
 * (elle a ete depose il y a longtemps)
 */
int fourmiliere::isPheroNear (int x0, int y0)
{
  float phero = 2000;
  int   dir   = -1;

  for (int d = 0; d < 8; d++)
    {
      int x = x0 + dirX[d];
      int y = y0 + dirY[d];
      
      if((x>=0) && (y>=0) && (x < sizex) && (y < sizey))
	{
	  if(hasPhero(x,y) && getPhero(x,y) < phero)
	    {
	      phero = getPhero(x,y);
	      dir   = d;
	    }
	}
    }
  return dir;
}
  
