/*
 * Projet LO43
 * Julien Miltat -- Julien Rosener -- Thomas Petazzoni
 * Semestre automne 2002-2003
 */

#ifndef __FOURMILIERE_H__
#define __FOURMILIERE_H__

#include "environnement.h"

class ListeFourmis;

class fourmiliere
{
 private:
  /* Reserve de nourriture de la fourmiliere */
  int   reserveNourriture;
  
  /* Taux d'evaporation des pheromones dans l'environnement */
  float tauxevapphero;
  
  /* Seuil a partir duquel on considere qu'il n'y a plus de
     pheromones */
  float zerophero;

  /* Taille de l'environnement */
  int   sizex, sizey;

  /* Position de la fourmiliere, nombre de fourmis */
  int   x, y, nbf;

  /* Numero de la fourmiliere */
  int   nf;

  /* Tableau bi-dimensionnel representant la quantite de pheromone
     pour cette fourmiliere, en tout point de l'environnement */
  float **phero;

  /* Pointeur vers l'environnement */
  environnement *env;

  /* Liste des fourmis */
  ListeFourmis *liste;

 public:
  fourmiliere(int x0, int y0, int nf0, 
	      int sizex0, int sizey0,
	      float tauxevapphero0, float zerophero0,
	      environnement *env0);
  ~fourmiliere();

  inline int getX() {return x;}
  inline int getY() {return y;}
  inline int getSizeX() { return sizex; }
  inline int getSizeY() { return sizey; }
  inline int getNbrFourmis() { return nbf;}
  inline int getFourmiliereId() { return nf; }
  inline int getNbrLarves() { return 0;}
  inline int getFood() { return reserveNourriture;}
  inline environnement * getEnv() { return env; }

  inline int   hasPhero(int x,int y) 
    { 
      return (phero[x][y] > zerophero); 
    }
  inline float getPhero(int x,int y) 
    { 
      return  phero[x][y];              
    }
  inline void  evapPhero(int x,int y) 
    { 
	int oldvalue = hasPhero(x, y);
	phero[x][y] *= tauxevapphero;

	/* On efface la pheromone seulement si il y en avait avant
	   l'evaporation */
	if(oldvalue && !hasPhero(x, y))
	  getEnv()->unshowPhero(x, y, nf);
    }
  inline void  addPhero(int x,int y, int d) 
    { 
      /* Maximum 1000 de pheromones */
      if(phero[x][y] > 1000)
	return;

      int oldvalue = hasPhero(x, y);
      phero[x][y] += d;
      
      /* On dessine la pheromone seulement si il n'y en avait pas
	 avant l'ajout */
      if(! oldvalue && hasPhero(x, y))
	getEnv()->showPhero(x, y, nf);
    }

  int  isPheroNear(int x0, int y0);
  void addFood(int food) { reserveNourriture += food; }
  void ajouter(fourmi &);
  void supprimer(fourmi &);
  void affiche();
  fourmi *get(int i);
  fourmi *getFirst();
  fourmi *getNext();
  void simu();
  int isDansNid (int x0, int y0); 
  void init(int x0, int y0, int nf);
  friend ostream& operator<< (ostream&, fourmiliere&);
  friend istream& operator>> (istream&, fourmiliere&);
};

#endif /* __FOURMILIERE_H__ */
